@@ -2,7 +2,7 @@
  * PL/R - PostgreSQL support for R as a
  *	      procedural language (PL)
  *
- * Copyright (c) 2003-2006 by Joseph E. Conway
+ * Copyright (c) 2003-2007 by Joseph E. Conway
  * ALL RIGHTS RESERVED
  * 
  * Joe Conway <mail@joeconway.com>
@@ -42,6 +42,7 @@ PG_MODULE_MAGIC;
  * Global data
  */
 MemoryContext plr_SPI_context;
+MemoryContext plr_caller_context;
 HTAB *plr_HashTable = (HTAB *) NULL;
 char *last_R_error_msg = NULL;
 
@@ -75,6 +76,8 @@ static Oid plr_nspOid = InvalidOid;
 #define THROWERROR_CMD \
 			"pg.throwerror <-function(msg) " \
 			"{stop(msg, call. = FALSE)}"
+#define OPTIONS_THROWWARN_CMD \
+			"options(warning.expression = expression(pg.thrownotice(last.warning)))"
 #define QUOTE_LITERAL_CMD \
 			"pg.quoteliteral <-function(sql) " \
 			"{.Call(\"plr_quote_literal\", sql)}"
@@ -89,6 +92,18 @@ static Oid plr_nspOid = InvalidOid;
 #define SPI_EXECP_CMD \
 			"pg.spi.execp <-function(sql, argvalues = NA) " \
 			"{.Call(\"plr_SPI_execp\", sql, argvalues)}"
+#define SPI_CURSOR_OPEN_CMD \
+			"pg.spi.cursor_open<-function(cursor_name,plan,argvalues=NA) " \
+			"{.Call(\"plr_SPI_cursor_open\",cursor_name,plan,argvalues)}"
+#define SPI_CURSOR_FETCH_CMD \
+			"pg.spi.cursor_fetch<-function(cursor,forward,rows) " \
+			"{.Call(\"plr_SPI_cursor_fetch\",cursor,forward,rows)}"
+#define SPI_CURSOR_MOVE_CMD \
+			"pg.spi.cursor_move<-function(cursor,forward,rows) " \
+			"{.Call(\"plr_SPI_cursor_move\",cursor,forward,rows)}"
+#define SPI_CURSOR_CLOSE_CMD \
+			"pg.spi.cursor_close<-function(cursor) " \
+			"{.Call(\"plr_SPI_cursor_close\",cursor)}"
 #define SPI_LASTOID_CMD \
 			"pg.spi.lastoid <-function() " \
 			"{.Call(\"plr_SPI_lastoid\")}"
@@ -112,6 +127,7 @@ static Oid plr_nspOid = InvalidOid;
 /*
  * static declarations
  */
+static void plr_atexit(void);
 static void plr_load_builtins(Oid funcid);
 static void plr_init_all(Oid funcid);
 static Datum plr_trigger_handler(PG_FUNCTION_ARGS);
@@ -144,6 +160,8 @@ plr_call_handler(PG_FUNCTION_ARGS)
 	MemoryContext	origcontext = CurrentMemoryContext;
 	/* save caller's SPI context */
 	MemoryContext	plr_SPI_context_save = plr_SPI_context;
+	/* save caller's context */
+	plr_caller_context = origcontext;
 
 	/* Connect to SPI manager */
 	if (SPI_connect() != SPI_OK_CONNECT)
@@ -262,6 +280,21 @@ PLR_CLEANUP
 	}
 }
 
+static void
+plr_atexit(void)
+{
+	/* only react during plr startup */
+	if (plr_pm_init_done)
+		return;
+
+	ereport(ERROR,
+			(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+			 errmsg("the R interpreter did not initialize"),
+			 errhint("R_HOME must be correct in the environment " \
+					 "of the user that starts the postmaster process.")));
+}
+
+
 /*
  * plr_init() - Initialize all that's safe to do in the postmaster
  *
@@ -272,7 +305,7 @@ plr_init(void)
 {
 	char	   *r_home;
 	int			rargc;
-	char	   *rargv[] = {"PL/R", "--silent", "--no-save"};
+	char	   *rargv[] = {"PL/R", "--silent", "--no-save", "--no-restore"};
 
 	/* refuse to init more than once */
 	if (plr_pm_init_done)
@@ -281,18 +314,63 @@ plr_init(void)
 	/* refuse to start if R_HOME is not defined */
 	r_home = getenv("R_HOME");
 	if (r_home == NULL)
-		ereport(ERROR,
-				(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
-				 errmsg("environment variable R_HOME not defined"),
-				 errhint("R_HOME must be defined in the environment " \
-						 "of the user that starts the postmaster process.")));
+	{
+		size_t		rh_len = strlen(R_HOME_DEFAULT);
+
+		/* see if there is a compiled in default R_HOME */
+		if (rh_len)
+		{
+			char	   *rhenv;
+			MemoryContext		oldcontext;
+
+			/* Needs to live until/unless we explicitly delete it */
+			oldcontext = MemoryContextSwitchTo(TopMemoryContext);
+			rhenv = palloc(8 + rh_len);
+			MemoryContextSwitchTo(oldcontext);
+
+			sprintf(rhenv, "R_HOME=%s", R_HOME_DEFAULT);
+			putenv(rhenv);
+		}
+		else
+			ereport(ERROR,
+					(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+					 errmsg("environment variable R_HOME not defined"),
+					 errhint("R_HOME must be defined in the environment " \
+							 "of the user that starts the postmaster process.")));
+	}
 
 	rargc = sizeof(rargv)/sizeof(rargv[0]);
-	Rf_initEmbeddedR(rargc, rargv);
+
+	/*
+	 * register an exit callback to handle the case where R does not initialize
+	 * and just exits with R_suicide()
+	 */
+	atexit(plr_atexit);
+
+	/*
+	 * When initialization fails, R currently exits. Check the return
+	 * value anyway in case this ever gets fixed
+	 */
+	if (!Rf_initEmbeddedR(rargc, rargv))
+		ereport(ERROR,
+				(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+				 errmsg("the R interpreter did not initialize"),
+				 errhint("R_HOME must be correct in the environment " \
+						 "of the user that starts the postmaster process.")));
 
 	/* arrange for automatic cleanup at proc_exit */
 	on_proc_exit(plr_cleanup, 0);
 
+#ifndef WIN32
+	/*
+	 * Force non-interactive mode since R may not do so.
+	 * See comment in Rembedded.c just after R_Interactive = TRUE:
+	 * "Rf_initialize_R set this based on isatty"
+	 * If Postgres still has the tty attached, R_Interactive remains TRUE
+	 */
+	R_Interactive = false;
+#endif
+
 	plr_pm_init_done = true;
 }
 
@@ -314,6 +392,7 @@ plr_load_builtins(Oid funcid)
 		OPTIONS_THROWRERROR_CMD,
 		THROWNOTICE_CMD,
 		THROWERROR_CMD,
+		OPTIONS_THROWWARN_CMD,
 
 		/* install the commands for SPI support in the interpreter */
 		QUOTE_LITERAL_CMD,
@@ -321,6 +400,10 @@ plr_load_builtins(Oid funcid)
 		SPI_EXEC_CMD,
 		SPI_PREPARE_CMD,
 		SPI_EXECP_CMD,
+		SPI_CURSOR_OPEN_CMD,
+		SPI_CURSOR_FETCH_CMD,
+		SPI_CURSOR_MOVE_CMD,
+		SPI_CURSOR_CLOSE_CMD,
 		SPI_LASTOID_CMD,
 		SPI_FACTOR_CMD,
 
@@ -694,9 +777,16 @@ compile_plr_function(FunctionCallInfo fcinfo)
 
 	if (function)
 	{
+		bool	function_valid;
+
 		/* We have a compiled function, but is it still valid? */
-		if (!(function->fn_xmin == HeapTupleHeaderGetXmin(procTup->t_data) &&
-			  function->fn_cmin == HeapTupleHeaderGetCmin(procTup->t_data)))
+		if (function->fn_xmin == HeapTupleHeaderGetXmin(procTup->t_data) &&
+			ItemPointerEquals(&function->fn_tid, &procTup->t_self))
+			function_valid = true;
+		else
+			function_valid = false;
+
+		if (!function_valid)
 		{
 			/*
 			 * Nope, drop the hashtable entry.  XXX someday, free all the
@@ -772,6 +862,7 @@ do_compile(FunctionCallInfo fcinfo,
 	StringInfo				proc_internal_args = makeStringInfo();
 	char				   *proc_source;
 	MemoryContext			oldcontext;
+	char				   *p;
 
 	/* grab the function name */
 	proname = NameStr(procStruct->proname);
@@ -801,7 +892,7 @@ do_compile(FunctionCallInfo fcinfo,
 
 	function->proname = pstrdup(proname);
 	function->fn_xmin = HeapTupleHeaderGetXmin(procTup->t_data);
-	function->fn_cmin = HeapTupleHeaderGetCmin(procTup->t_data);
+	function->fn_tid = procTup->t_self;
 
 	/* Lookup the pg_language tuple by Oid*/
 	langTup = SearchSysCache(LANGOID,
@@ -1162,13 +1253,34 @@ do_compile(FunctionCallInfo fcinfo,
 		elog(ERROR, "null prosrc");
 	proc_source = DatumGetCString(DirectFunctionCall1(textout, prosrcdatum));
 
-	appendStringInfo(proc_internal_def, "%s}", proc_source);
+	/*
+	 * replace any carriage returns with either a space or a newline,
+	 * as appropriate
+	 */
+	p = proc_source;
+	while (*p != '\0')
+	{
+		if (p[0] == '\r')
+		{
+			if (p[1] == '\n')
+				/* for crlf sequence, write over the cr with a space */
+				*p++ = ' ';
+			else
+				/* otherwise write over the cr with a nl */
+				*p++ = '\n';
+		}
+		else
+			p++;
+	}
 
 	/* parse or find the R function */
 	if(proc_source && proc_source[0])
-		function->fun = plr_parse_func_body(proc_internal_def->data);
+		appendStringInfo(proc_internal_def, "%s}", proc_source);
 	else
-		function->fun = Rf_findFun(Rf_install(proname), R_GlobalEnv);
+		appendStringInfo(proc_internal_def, "%s(%s)}",
+						 function->proname,
+						 proc_internal_args->data);
+	function->fun = plr_parse_func_body(proc_internal_def->data);
 
 	R_PreserveObject(function->fun);
 
@@ -1201,15 +1313,21 @@ plr_parse_func_body(const char *body)
 {
 	SEXP	rbody;
 	SEXP	fun;
+    SEXP	tmp;
 	int		status;
 
 	PROTECT(rbody = NEW_CHARACTER(1));
 	SET_STRING_ELT(rbody, 0, COPY_TO_USER_STRING(body));
 
-	PROTECT(fun = VECTOR_ELT(R_PARSEVECTOR(rbody, -1, &status), 0));
+	tmp = PROTECT(R_PARSEVECTOR(rbody, -1, &status));
+	if (tmp != R_NilValue)
+		PROTECT(fun = VECTOR_ELT(tmp, 0));
+	else
+		PROTECT(fun = R_NilValue);
+
 	if (status != PARSE_OK)
 	{
-		UNPROTECT(2);
+		UNPROTECT(3);
 		if (last_R_error_msg)
 			ereport(ERROR,
 					(errcode(ERRCODE_DATA_EXCEPTION),
@@ -1223,7 +1341,7 @@ plr_parse_func_body(const char *body)
 							   "in \"%s\".", body)));
 	}
 
-	UNPROTECT(2);
+	UNPROTECT(3);
 	return(fun);
 }
 
@@ -1247,7 +1365,12 @@ call_r_func(SEXP fun, SEXP rargs)
 			obj = CDR(obj);
 		}
 		UNPROTECT(1);
-		PROTECT(call = lcons(fun, args));
+        /*
+         * NB: the headers of both R and Postgres define a function
+         * called lcons, so use the full name to be precise about what
+         * function we're calling.
+         */
+		PROTECT(call = Rf_lcons(fun, args));
 	}
 	else
 	{
@@ -1314,7 +1437,7 @@ plr_convertargs(plr_function *function, Datum *arg, bool *argnull)
 		else
 		{
 			/* better be a pg array arg, convert to a multi-row vector */
-			Datum		dvalue = arg[i];
+			Datum		dvalue = (Datum) PG_DETOAST_DATUM(arg[i]);
 			FmgrInfo	out_func = function->arg_elem_out_func[i];
 			int			typlen = function->arg_elem_typlen[i];
 			bool		typbyval = function->arg_elem_typbyval[i];
@@ -1478,3 +1601,19 @@ getModulesSql(Oid nspOid)
 
     return sql->data;
 }
+
+#ifdef DEBUGPROTECT
+SEXP
+pg_protect(SEXP s, char *fn, int ln)
+{
+	elog(NOTICE, "\tPROTECT\t1\t%s\t%d", fn, ln);
+	return protect(s);
+}
+
+void
+pg_unprotect(int n, char *fn, int ln)
+{
+	elog(NOTICE, "\tUNPROTECT\t%d\t%s\t%d", n, fn, ln);
+	unprotect(n);
+}
+#endif /* DEBUGPROTECT */
