@@ -2,7 +2,7 @@
  * PL/R - PostgreSQL support for R as a
  *	      procedural language (PL)
  *
- * Copyright (c) 2003-2006 by Joseph E. Conway
+ * Copyright (c) 2003-2007 by Joseph E. Conway
  * ALL RIGHTS RESERVED
  * 
  * Joe Conway <mail@joeconway.com>
@@ -36,9 +36,50 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <setjmp.h>
+#include <stdlib.h>
 #include <sys/stat.h>
 
-#define ELOG_H
+#include "R.h"
+#include "Rversion.h"
+/*
+ * R version is calculated thus:
+ *   Maj * 65536 + Minor * 256 + Build * 1
+ * So:
+ * version 1.8.0 results in:
+ *   (1 * 65536) + (8 * 256) + (0 * 1) == 67584
+ * version 1.9.0 results in:
+ *   (1 * 65536) + (9 * 256) + (0 * 1) == 67840
+ */
+#if (R_VERSION >= 132096) /* R_VERSION >= 2.4.0 */
+#include "Rembedded.h"
+#endif
+#ifndef WIN32
+#include "Rinterface.h"
+#endif
+#include "Rinternals.h"
+#include "Rdefines.h"
+#include "Rdevices.h"
+
+/* starting in R-2.7.0 this defn was removed from Rdevices.h */
+#ifndef KillAllDevices
+#define KillAllDevices					Rf_KillAllDevices
+#endif
+
+/*
+ * The R headers define various symbols that are also defined by the
+ * Postgres headers, so undef them first to avoid conflicts.
+ */
+#ifdef ERROR
+#undef ERROR
+#endif
+
+#ifdef WARNING
+#undef WARNING
+#endif
+
+#ifdef lcons
+#undef lcons
+#endif
 
 #include "postgres.h"
 
@@ -60,43 +101,23 @@
 #include "utils/lsyscache.h"
 #include "utils/syscache.h"
 
-#include "R.h"
-#include "Rinternals.h"
-#include "Rdefines.h"
-#include "Rdevices.h"
-#include "Rversion.h"
-
-#ifdef ERROR
-#undef ERROR
-#endif
-
-#ifdef WARNING
-#undef WARNING
-#endif
-
-#undef ELOG_H
-#include "utils/elog.h"
-
 /* working with postgres 7.3 compatible sources */
-#if (CATALOG_VERSION_NO <= 200510211)
-#error "This version of PL/R only builds with PostgreSQL 8.2"
-#else
+#if !defined(PG_VERSION_NUM) || PG_VERSION_NUM < 80200
+#error "This version of PL/R only builds with PostgreSQL 8.2 or later"
+#elif PG_VERSION_NUM < 80300
 #define PG_VERSION_82_COMPAT
+#else
+#define PG_VERSION_83_COMPAT
 #endif
 
 #ifdef DEBUGPROTECT
 #undef PROTECT
-#define PROTECT(s) \
-	do { \
-		elog(NOTICE, "\tPROTECT\t1\t%s\t%d", __FILE__, __LINE__); \
-		protect(s); \
-	} while (0)
+extern SEXP pg_protect(SEXP s, char *fn, int ln);
+#define PROTECT(s)		pg_protect(s, __FILE__, __LINE__)
+
 #undef UNPROTECT
-#define UNPROTECT(n) \
-	do { \
-		elog(NOTICE, "\tUNPROTECT\t%d\t%s\t%d", n, __FILE__, __LINE__); \
-		unprotect(n); \
-	} while (0)
+extern void pg_unprotect(int n, char *fn, int ln);
+#define UNPROTECT(n)	pg_unprotect(n, __FILE__, __LINE__)
 #endif /* DEBUGPROTECT */
 
 #define xpfree(var_) \
@@ -114,23 +135,8 @@
 		xpfree(mystr_); \
 	} while (0)
 
-#define resetStringInfo(mystr_) \
-	do { \
-		xpfree((mystr_)->data); \
-		initStringInfo(mystr_); \
-	} while (0)
-
 #define NEXT_STR_ELEMENT	" %s"
 
-/*
- * R version is calculated thus:
- *   Maj * 65536 + Minor * 256 + Build * 1
- * So:
- * version 1.8.0 results in:
- *   (1 * 65536) + (8 * 256) + (0 * 1) == 67584
- * version 1.9.0 results in:
- *   (1 * 65536) + (9 * 256) + (0 * 1) == 67840
- */
 #if (R_VERSION < 67840) /* R_VERSION < 1.9.0 */
 #define SET_COLUMN_NAMES \
 	do { \
@@ -182,9 +188,13 @@ extern void R_RunExitFinalizers(void);
 #else /* R_VERSION >= 1.8.0 */
 
 #include "R_ext/Parse.h"
-#define R_PARSEVECTOR(a_, b_, c_)		R_ParseVector(a_, b_, (ParseStatus *) c_)
 
-#endif /* R_VERSION */
+#if (R_VERSION >= 132352) /* R_VERSION >= 2.5.0 */
+#define R_PARSEVECTOR(a_, b_, c_)		R_ParseVector(a_, b_, (ParseStatus *) c_, R_NilValue)
+#else /* R_VERSION < 2.5.0 */
+#define R_PARSEVECTOR(a_, b_, c_)		R_ParseVector(a_, b_, (ParseStatus *) c_)
+#endif /* R_VERSION >= 2.5.0 */
+#endif /* R_VERSION >= 1.8.0 */
 
 /* convert C string to text pointer */
 #define PG_TEXT_GET_STR(textp_) \
@@ -213,6 +223,7 @@ extern void R_RunExitFinalizers(void);
 
 #define POP_PLERRCONTEXT \
 	do { \
+		pfree(plerrcontext.arg); \
 		error_context_stack = plerrcontext.previous; \
 	} while (0)
 
@@ -238,7 +249,7 @@ extern void R_RunExitFinalizers(void);
 
 #define INIT_AUX_FMGR_ATTS \
 	do { \
-		finfo->fn_mcxt = QueryContext; \
+		finfo->fn_mcxt = plr_caller_context; \
 		finfo->fn_expr = (Node *) NULL; \
 	} while (0)
 
@@ -380,7 +391,7 @@ typedef struct plr_function
 {
 	char			   *proname;
 	TransactionId		fn_xmin;
-	CommandId			fn_cmin;
+	ItemPointerData		fn_tid;
 	plr_func_hashkey   *fn_hashkey; /* back-link to hashtable key */
 	bool				lanpltrusted;
 	Oid					result_typid;
@@ -445,6 +456,10 @@ extern SEXP plr_quote_ident(SEXP rawstr);
 extern SEXP plr_SPI_exec(SEXP rsql);
 extern SEXP plr_SPI_prepare(SEXP rsql, SEXP rargtypes);
 extern SEXP plr_SPI_execp(SEXP rsaved_plan, SEXP rargvalues);
+extern SEXP plr_SPI_cursor_open(SEXP cursor_name_arg,SEXP rsaved_plan, SEXP rargvalues);
+extern SEXP plr_SPI_cursor_fetch(SEXP cursor_in,SEXP forward_in, SEXP rows_in);
+extern void plr_SPI_cursor_close(SEXP cursor_in);
+extern void plr_SPI_cursor_move(SEXP cursor_in, SEXP forward_in, SEXP rows_in);
 extern SEXP plr_SPI_lastoid(void);
 extern void throw_r_error(const char **msg);
 
@@ -455,6 +470,8 @@ extern Datum plr_array_push(PG_FUNCTION_ARGS);
 extern Datum plr_array(PG_FUNCTION_ARGS);
 extern Datum plr_array_accum(PG_FUNCTION_ARGS);
 extern Datum plr_environ(PG_FUNCTION_ARGS);
+extern Datum plr_set_rhome(PG_FUNCTION_ARGS);
+extern Datum plr_unset_rhome(PG_FUNCTION_ARGS);
 
 /* Postgres backend support functions */
 extern void compute_function_hashkey(FunctionCallInfo fcinfo,
